<?php

declare(strict_types=1);

namespace App\Controller\Admin\Ajax;

use App\Entity\AgentProfile\AgentProfile;
use App\Entity\ConfigProfile;
use App\Entity\Device;
use App\Entity\DeviceOsVersion;
use App\Entity\Label;
use App\Entity\LabelDeviceMap;
use App\Entity\MonitoringGroup;
use App\Entity\MonitoringGroupMapping;
use App\Repository\DeviceRepository;
use App\Service\Api\AgentProfiles;
use Doctrine\DBAL\Exception;
use Doctrine\DBAL\Exception\UniqueConstraintViolationException;
use Doctrine\Persistence\ManagerRegistry;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController as Controller;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;

/**
 * @Route("/admin/ajax/device", name="ajax_device_")
 */
class DeviceController extends Controller
{

    /**
     * Get devices not assigned to web tunnels
     * @Route("/not-assigned-web-tunnel", methods={"GET"}, name="get_not_web_tunnel")
     * @param Request $request
     * @param DeviceRepository $deviceRepository
     * @return JsonResponse|Response
     * @throws Exception
     */
    public function getAllDevices(Request $request, DeviceRepository $deviceRepository): Response {

        if ($request->isXmlHttpRequest()) {

            $devices = $deviceRepository->findAllNotAssignedToWebTunnelPlainSql();
            $result = array();

            foreach ($devices as $device) {
                $result[$device['id_device']] = ['value' => $device['devid'], 'description' => $device['name']];
            }

            return new JsonResponse($result);
        }

        return new Response('Bad request.', 400);

    }

    /**
     * Get all Devices not in Monitoring Group.
     * @Route("/get/all-not/{mntGroupId}", methods={"GET"}, name="get_all_not_in_monitoring_group")
     * @param Request $request
     * @param DeviceRepository $deviceRepository
     * @param int $mntGroupId
     * @return Response
     */
    public function getAllDevicesNotInMonitoringGroup(Request          $request,
                                                      DeviceRepository $deviceRepository,
                                                      int $mntGroupId): Response {

        if ($request->isXmlHttpRequest()) {

            $devices = $deviceRepository->findAllNotInMonitoringGroupPlainSql($mntGroupId);

            $result = array();

            foreach ($devices as $device) {
                $result[$device['id_device']] = ['value' => $device['devid'], 'description' => $device['name']];
            }

            return new JsonResponse($result);
        }

        return new Response('Bad request.', 400);

    }

    /**
     * Add Devices to Monitoring Groups
     * @Route("/add/monitoring-group", methods={"POST"}, name="add_to_monitoring_group")
     * @param Request $request
     * @param ManagerRegistry $managerRegistry
     * @return Response
     */
    public function addDevicesToMonitoringGroupAction(Request $request, ManagerRegistry $managerRegistry): Response
    {

        if (!$request->isXmlHttpRequest()) {

            return new Response('Bad request.', 400);

        }

        if (isset($request->request)) {

            $monitoringGroups = (array)$request->get('monitoringGroups');
            $selectedDevices = $request->get('selectedDevices');

            if (!isset($monitoringGroups) && !isset($selectedDevices))
                return new JsonResponse(array('status' => 'Error'), 400);

            $monitoringGroups = $managerRegistry->getRepository(MonitoringGroup::class)->findBy(['id' => $monitoringGroups]);
            $deviceRepository = $managerRegistry->getRepository(Device::class);

            foreach ($selectedDevices as $device) {

                $device = $deviceRepository->findOneBy(['id' => $device['id']]);

                foreach ($monitoringGroups as $monitoringGroup) {

                    $existingGroupMapping = $managerRegistry->getRepository(MonitoringGroupMapping::class)
                        ->findOneBy(['device' => $device, 'monitoringGroup' => $monitoringGroup]);

                    if(!$existingGroupMapping){

                        try {

                            $monitoringGroupMapping = new MonitoringGroupMapping();
                            $monitoringGroupMapping->setMonitoringGroup($monitoringGroup);
                            $monitoringGroupMapping->setDevice($device);
                            $managerRegistry->getManager()->persist($monitoringGroupMapping);
                            $managerRegistry->getManager()->flush();

                        } catch (UniqueConstraintViolationException $e) {

                            //Do nothing, label already assigned

                        } catch (\Exception $e) {

                        }

                    }
                }
            }

            return new JsonResponse(array(
                'code' => 200,
                'message' => 'Devices were successfully assigned to Monitoring Groups.',
                'errors' => array('errors' => array(''))),
                200);

        }

        // If we reach this point, it means that something went wrong
        return new JsonResponse(array(
            'status' => 'Error',
            'message' => 'Something bad happened.'),
            400);

    }

    /**
     * Add Labels to Devices
     * @Route("/add/label", methods={"POST"}, name="add_label")
     * @param Request $request
     * @param ManagerRegistry $managerRegistry
     * @return Response
     */
    public function addLabelToDeviceAction(Request $request, ManagerRegistry $managerRegistry): Response {

        if (!$request->isXmlHttpRequest()) {

            return new Response('Bad request.', 400);

        }

        if (isset($request->request)) {

            $labels = (array)$request->get('labels');
            $selectedDevices = $request->get('selectedDevices');

            if (!isset($labels) && !isset($selectedDevices))
                return new JsonResponse(array('status' => 'Error'), 400);

            $labels = $managerRegistry->getRepository(Label::class)->findBy(['id' => $labels]);
            $deviceRepository = $managerRegistry->getRepository(Device::class);
            foreach ($selectedDevices as $device) {

                $device = $deviceRepository->findOneBy(['id' => $device['id']]);

                foreach ($labels as $label) {

                    $existingLabel = $managerRegistry->getRepository(LabelDeviceMap::class)
                        ->findOneBy(['device' => $device, 'label' => $label]);

                    if(!$existingLabel){

                        try {

                            $labelDeviceMapping = new LabelDeviceMap();
                            $labelDeviceMapping->setLabel($label);
                            $labelDeviceMapping->setDevice($device);
                            $managerRegistry->getManager()->persist($labelDeviceMapping);
                            $managerRegistry->getManager()->flush();

                        } catch (UniqueConstraintViolationException $e) {

                            //Do nothing, label already assigned

                        } catch (\Exception $e) {

                        }

                    }

                }

            }

            return new JsonResponse(array(
                'code' => 200,
                'message' => 'Label was successfully assigned to the devices.',
                'errors' => array('errors' => array(''))),
                200);

        }

        // If we reach this point, it means that something went wrong
        return new JsonResponse(array(
            'status' => 'Error',
            'message' => 'Something bad happened.'),
            400);

    }

    /**
     * Add Fsw to Devices
     * @Route("/add/fw", methods={"POST"}, name="add_fw")
     * @param Request $request
     * @param ManagerRegistry $managerRegistry
     * @return Response
     */
    public function addFwToDeviceAction(Request $request, ManagerRegistry $managerRegistry): Response {

        if (!$request->isXmlHttpRequest()) {

            return new Response('Bad request.', 400);

        }

        if (isset($request->request)) {

            $fws = (array)$request->get('fws');
            $selectedDevices = $request->get('selectedDevices');

            if (!isset($fws) && !isset($selectedDevices))
                return new JsonResponse(array('status' => 'Error'), 400);

            $fws = $managerRegistry->getRepository(DeviceOsVersion::class)->findBy(['id' => $fws]);
            $deviceRepository = $managerRegistry->getRepository(Device::class);

            $errors = [];
            $success = [];

            foreach ($selectedDevices as $device) {

                $device = $deviceRepository->findOneBy(['id' => $device['id']]);

                $deviceType = $device->getDevType();

                foreach ($fws as $fw) {

                    $fwDeviceOsType = $fw->getOsType()->getDevType();

                    //Check if there is the same OS type on device as selected FW
                    if ($deviceType === $fwDeviceOsType) {

                        try {

                            $device->setOsVerDesired($fw);
                            $managerRegistry->getManager()->persist($device);
                            $managerRegistry->getManager()->flush();

                        } catch (\Exception $e) {

                        }

                        $success[] = $device->getDevId();

                    } else {

                        $errors[] = $device->getDevId();

                    }

                }

            }

            return new JsonResponse([
                'code' => 200,
                'message' => 'Fw version successfully assigned to the devices: ',
                'success' => $success,
                'errors' => $errors
            ],
                200);

        }

        // If we reach this point, it means that something went wrong
        return new JsonResponse(array(
            'status' => 'Error',
            'message' => 'Something bad happened.'),
            400);

    }

    /**
     * Add Agent Profile to Devices
     * @Route("/add/agent-profile", methods={"POST"}, name="add_agent_profile")
     * @param Request $request
     * @param ManagerRegistry $managerRegistry
     * @return Response
     */
    public function addAgentProfileToDeviceAction(Request $request, ManagerRegistry $managerRegistry,
                                                  AgentProfiles $agentProfilesApi): Response {

        if (!$request->isXmlHttpRequest()) {

            return new Response('Bad request.', 400);

        }

        if (isset($request->request)) {

            $agentProfiles = (array)$request->get('agentProfiles');
            $selectedDevices = $request->get('selectedDevices');

            if (!isset($agentProfiles) && !isset($selectedDevices))
                return new JsonResponse(array('status' => 'Error'), 400);

            $agentProfiles = $managerRegistry->getRepository(AgentProfile::class)->findBy(['id' => $agentProfiles]);
            $deviceRepository = $managerRegistry->getRepository(Device::class);

            $errors = [];
            $success = [];

            foreach ($agentProfiles as $agentProfile) {

                $deviceIds = [];
                $agentProfileId = $agentProfile->getId();

                foreach ($selectedDevices as $device) {

                    $device = $deviceRepository->findOneBy(['id' => $device['id']]);
                    $deviceIds[] = $device->getId();

                }

                try {

                    $result = $agentProfilesApi->assignDeviceAgentProfile($agentProfileId, $deviceIds);
                    if($result->operation_result->Code < 0){
                        throw new \Exception('Cannot call api');
                    }
                    /*$device->setAgentProfile($agentProfile);
                    $managerRegistry->getManager()->persist($device);
                    $managerRegistry->getManager()->flush();*/

                    $success[] = 'Selected profile was assigned.'; //$device->getDevId();

                } catch (\Exception $e) {

                    $errors[] = 'Something bad happend.' . $e->getMessage(); //$device->getDevId();

                }

            }

            return new JsonResponse([
                'code' => 200,
                'message' => 'Agent Profile was successfully assigned to the devices: ',
                'success' => $success,
                'errors' => $errors
            ],
                200);

        }

        // If we reach this point, it means that something went wrong
        return new JsonResponse(array(
            'status' => 'Error',
            'message' => 'Something bad happened.'),
            400);

    }

    /**
     * Get Devices with Config Profile
     * @Route("/get/config-profiles", methods={"GET"}, name="get_config_profiles")
     * @param Request $request
     * @param DeviceRepository $deviceRepository
     * @param ManagerRegistry $managerRegistry
     * @return Response
     * @throws Exception
     */
    public function getConfigProfilesAction(Request $request, DeviceRepository $deviceRepository, ManagerRegistry $managerRegistry): Response
    {

        if ($request->isXmlHttpRequest()) {

            $devices = $deviceRepository->getDevicesToClonedFrom();

            $result = array();

            foreach ($devices as $device) {
                $result[$device['id_device']] = ['value' => $device['devid'], 'description' => $device['name']];
            }

            return new JsonResponse($result);
        }

        return new JsonResponse(array(
            'status' => 'Error',
            'message' => 'Something bad happened.'),
            400);

    }

    /**
     * Get All Agent Profiles
     * @Route("/get/agent-profiles", methods={"GET"}, name="get_agent_profiles")
     * @param Request $request
     * @param ManagerRegistry $managerRegistry
     * @return Response
     * @throws Exception
     */
    public function getAgentProfilesAction(Request $request, ManagerRegistry $managerRegistry): Response
    {

        if ($request->isXmlHttpRequest()) {

            $agentProfiles = $managerRegistry->getRepository(AgentProfile::class)->findAll();

            $result = array();

            foreach ($agentProfiles as $agentProfile) {
                $result[$agentProfile->getId()] = ['value' => $agentProfile->getProfileName(), 'description' => ''];
            }

            return new JsonResponse($result);
        }

        return new JsonResponse(array(
            'status' => 'Error',
            'message' => 'Something bad happened.'),
            400);

    }

    /**
     * Add Config Profile to Devices
     * @Route("/add/config-profile", methods={"POST"}, name="add_config_profile")
     * @param Request $request
     * @param ManagerRegistry $managerRegistry
     * @return Response
     */
    public function addConfigProfileToDeviceAction(Request $request, ManagerRegistry $managerRegistry): Response {

        if (!$request->isXmlHttpRequest()) {

            return new Response('Bad request.', 400);

        }

        if (isset($request->request)) {

            $devices = (array)$request->get('devices');
            $selectedDevices = $request->get('selectedDevices');
            $overwrite = $request->get('overwrite'); //retype to bool did not work for false

            if (!isset($devices) && !isset($selectedDevices))
                return new JsonResponse(array('status' => 'Error'), 400);

            $devices = $managerRegistry->getRepository(Device::class)->findBy(['id' => $devices]);
            $deviceRepository = $managerRegistry->getRepository(Device::class);

            $errors = [];
            $success = [];

            foreach ($selectedDevices as $device) {

                $device = $deviceRepository->findOneBy(['id' => $device['id']]);

                $deviceType = $device->getDevType();

                foreach ($devices as $deviceCloneFrom) {

                    $deviceTypeCloneFrom = $deviceCloneFrom->getDevType();
                    $deviceConfigProfileFrom = $deviceCloneFrom->getConfigProfileAutomatic();

                    //Check if there is the same OS type on device as selected FW
                    if (($deviceType === $deviceTypeCloneFrom && $device !== $deviceCloneFrom) && (
                        ($device->getConfigProfile() instanceof ConfigProfile && $overwrite == 'true') ||
                        ($device->getConfigProfile() === null)) ) {

                        try {

                            $device->setConfigProfile($deviceConfigProfileFrom);
                            $managerRegistry->getManager()->persist($device);
                            $managerRegistry->getManager()->flush();

                        } catch (\Exception $e) {

                        }

                        $success[] = $device->getDevId();

                    } else {

                        $errors[] = $device->getDevId();

                    }

                }

            }

            return new JsonResponse([
                'code' => 200,
                'message' => 'Config Profile was successfully assigned to the devices: ',
                'success' => $success,
                'errors' => $errors
            ],
                200);

        }

        // If we reach this point, it means that something went wrong
        return new JsonResponse(array(
            'status' => 'Error',
            'message' => 'Something bad happened.'),
            400);

    }

    /**
     * Remove Config Profile for Devices
     * @Route("/remove/config-profile", methods={"POST"}, name="remove_config_profile")
     * @param Request $request
     * @param ManagerRegistry $managerRegistry
     * @param DeviceRepository $deviceRepository
     * @return Response
     */
    public function removeConfigProfileDeviceAction(Request $request, ManagerRegistry $managerRegistry,
                                                    DeviceRepository $deviceRepository): Response
    {

        if (!$request->isXmlHttpRequest()) {

            return new Response('Bad request.', 400);

        }

        if (isset($request->request)) {

            $selectedDevices = $request->get('selectedDevices');

            if (!isset($selectedDevices))
                return new JsonResponse(array('status' => 'Error'), 400);

            $errors = [];
            $success = [];

            foreach ($selectedDevices as $device) {

                $device = $deviceRepository->findOneBy(['id' => $device['id']]);

                $device->setConfigProfile(null);
                $managerRegistry->getManager()->persist($device);
                $managerRegistry->getManager()->flush();

            }

            return new JsonResponse([
                'code' => 200,
                'message' => 'Config Profile was successfully removed for selected devices.',
                'success' => $success,
                'errors' => $errors
            ],
                200);

        }
    }

}